# DEPRECATION: This project was only created for a Proof of Concept purpose. The production ready version of this project received the name of **AWS Data Wrangler** (pip install awswrangler).
### Please consider move forward to:
* https://pypi.org/project/awswrangler/
* https://github.com/awslabs/aws-data-wrangler


------------------------------------------------------------------------------------------------------------


[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/ambv/black)

# PandasGlue

***A Python library for creating lite ETLs with the widely used Pandas library and the power of AWS Glue Catalog.***


With **PandasGLue** you will be able to write/read to/from an AWS Data Lake with one single line of code. With its minimalist nature **PandasGLue** has an interface with only 2 functions:


|   function   	|       From       	|        To        	|
|:------------:	|:----------------:	|:----------------:	|
| write_glue() 	| Pandas DataFrame 	|  AWS Glue Table  	|
|  read_glue() 	|   AWS GlueTable  	| Pandas DataFrame 	|



Once your data is mapped to [AWS Glue Catalog](https://aws.amazon.com/glue/) it will be accessible to many other tools like [AWS Redshift Spectrum](https://aws.amazon.com/redshift/), [AWS Athena](https://aws.amazon.com/athena/), [AWS Glue Jobs](https://aws.amazon.com/glue/), [AWS EMR](https://aws.amazon.com/emr/) ([Spark](https://spark.apache.org/), [Hive](https://hive.apache.org/), [PrestoDB](https://prestodb.github.io)), etc.

[Amazon Glue](https://aws.amazon.com/glue/) is an [AWS](https://aws.amazon.com/) simple, flexible, and cost-effective ETL service and [Pandas](https://pandas.pydata.org/) is a Python library which provides high-performance, easy-to-use data structures and data analysis tools.

The goal of this package is help data engineers in the usage of cost efficient serverless compute services ([Lambda](https://aws.amazon.com/glue/), [Glue](https://aws.amazon.com/lambda/), [Athena](https://aws.amazon.com/athena/)) in order to provide an easy way to integrate Pandas with  AWS Glue,  allowing load (*appending, overwriting or only overwriting the partitions with data*) the content of a DataFrame (**Write function**) directly in a table (parquet/csv format) in the [Glue Data Catalog](https://docs.aws.amazon.com/glue/latest/dg/populate-data-catalog.html) and also execute Athena queries (**Read function**) returning the result directly in a Pandas DataFrame.

## Use cases

This package is recommended for ETL purposes which loads and transforms small to medium size datasets without requiring to create Spark jobs, helping reduce infrastructure costs.

It could be used within [Lambda functions](https://docs.aws.amazon.com/lambda/latest/dg/lambda-introduction-function.html), [Glue scripts](https://docs.aws.amazon.com/glue/latest/dg/aws-glue-programming-python.html), [EC2](https://aws.amazon.com/ec2/) instances or any other infrastucture resources.

<p align="center">
  <img src="https://github.com/andresmao/test/blob/master/Pandas_glue_workflow2.png" width="700"  title="ETL Workflow">
</p>

### Prerequisites

* [Python 2.7, 3.6, 3.7](https://www.python.org/downloads/) 

```
pip install pandas
pip install boto3
pip install pyarrow 
```

### Installing the package...

```
pip install pandasglue
```

Or you can download direct the artifacts for [AWS Lambda Layer](https://docs.aws.amazon.com/lambda/latest/dg/configuration-layers.html) / [AWS Glue Job](https://docs.aws.amazon.com/glue/latest/dg/add-job-python.html) from our [release page](https://github.com/igorborgest/pandasglue/releases). Then you only will need to upload it in your AWS account.

## Usage 

**Read method:**

***read_glue()***

To retrieve the result of an Athena Query in a Pandas DataFrame.

Quick example:

```python
import pandas as pd
import pandasglue as pg

#Parameters
sql_query = "SELECT * FROM table_name LIMIT 20" 
db_name = "DB_NAME"
s3_output_bucket = "s3://bucket-url/"

df = pg.read_glue(sql_query,db_name,s3_output_bucket)

print(df)

```
***Parameters list:***

* ***query:*** the SQL statement on Athena
* ***db:*** database name.
* ***s3_output:*** path of the S3 output folder (optional)
* ***region:*** id of the AWS region, e.g: us-west-1 (optional)
* ***key:*** AWS Access key (optional)
* ***secret:*** AWS secret key (optional)
* ***profile_name:*** AWS IAM profile (optional)

<p>
  <hr>
</p>


**Write method:**

***write_glue()***

Convert a given Pandas Dataframe to a Glue Parquet table.

Quick example:

```python
import pandas as pd
import pandasglue as pg

#Parameters
database = "DB_NAME"
table_name = "TB_NAME"
s3_path = "s3://bucket-url/"

#Sample DF
source_data = {'name': ['Sarah', 'Renata', 'Erika', 'Fernanda', 'Diana'], 
        'city': ['Seattle', 'Sao Paulo', 'Seattle', 'Santiago', 'Lima'],
         'test_score': [82, 52, 56, 234, 254]}
         
df = pd.DataFrame(source_data, columns = ['name', 'city', 'test_score'])


pg.write_glue(df, database, table_name, s3_path, partition_cols=['city'])
```


***Parameters list:***

* ***df:*** variable containing the Pandas DataFrame
* ***database:*** database name.
* ***path:*** Path of the target S3 bucket
* ***table:*** table name (optional)
* ***partition_cols:*** list of columns to partition (optional)
* ***preserve_index:*** boolean, if you want to preserve the index on the table (optional)
* ***file_format:*** parquet|csv(optional)
* ***mode:*** append|overwrite|overwrite_partitions(optional)
* ***region:*** ID of the AWS region (optional)
* ***key:*** AWS Access key (optional)
* ***secret:*** AWS secret key (optional)
* ***profile_name:*** AWS IAM profile (optional)


## Built With

* [Boto3](https://boto3.amazonaws.com/v1/documentation/api/latest/index.html) - (AWS) SDK for Python, which allows Python developers to write software that makes use of Amazon services like S3 and EC2.
* [PyArrow](https://pypi.org/project/pyarrow/) - Python package to interoperate Arrow with Python allowing to convert text files format to parquet files among other functions.


## Contributing

Please read [CONTRIBUTING.md](https://gist.github.com/) for details on our code of conduct, and the process for submitting pull requests to us.

## Authors

* *Igor Tavares* - [Profile link](https://github.com/igorborgest)
* *Ricardo Serafim* - [Profile link](https://github.com/rcserafim)
* *Andres Palacios* - [Profile link](https://github.com/andresmao)

See also the list of [contributors](https://github.com/your/project/contributors) who participated in this project.

## License

This project is licensed under the MIT License - see the [LICENSE.md](LICENSE.md) file for details


